import redis

from rosetta_dispatcher.singleton import Singleton


class RedisClient(metaclass=Singleton):
    def __init__(self, host, port, password=None, decode_responses=True):
        self.pool = redis.ConnectionPool(host=host, port=port, password=password, decode_responses=decode_responses,
                                         socket_connect_timeout=5, socket_timeout=90, health_check_interval=90)

    @property
    def conn(self):
        return redis.Redis(connection_pool=self.pool)


from time import sleep

from rosetta_dispatcher.dispatch_server import DispatchServer
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel
from rosetta_dispatcher.model.dispatch_types import DispatchResponseStatus

host = '10.130.64.136'
# host = '121.11.219.106'
port = 4012

# host = '123.57.176.234'
# port = 6379
passwd = None

ds = DispatchServer(redis_host=host, redis_port=port, password=passwd)

count = 0
while True:
    # result = ds.fetch('BATCH_WORKER_QUEUE_DEV', batch_count=128)
    result = ds.fetch('WSD_BATCH_RPC_QUEUE_DEV', batch_count=16)
    print(f'fetched: {len(result)}')
    response_list = [DispatchResponseModel(correlation_id=request.correlation_id, status=DispatchResponseStatus.OK,
                                         data=request.data) for request in result]

    ds.batch_send_response(result, response_list)





#!/usr/bin/python3
import json
from datetime import datetime

from fastapi import FastAPI, Response, status

from rosetta_dispatcher.aio_dispatch_client import AioDispatchClient
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel
from rosetta_dispatcher.model.dispatch_types import DispatchResponseStatus

app = FastAPI()
dc: AioDispatchClient = None


@app.get("/")
async def root(response: Response):
    # return "hello world"
    data = {
        "source_language": "zh",
        "target_language": "en",
        "terminology_list": [
        ],
        "chapter_content": [
            ""
        ]
    }

    # print('before process')
    start = datetime.utcnow()
    result: DispatchResponseModel = await dc.process(service_queue='BATCH_WORKER_QUEUE_DEV', data=json.dumps(data), timeout=60)
    end = datetime.utcnow()
    if not result:
        print(end - start, result)

    if result.status == DispatchResponseStatus.OK:
        return result.data
    else:
        response.status_code = status.HTTP_500_INTERNAL_SERVER_ERROR



@app.on_event("startup")
async def init_service():
    global dc
    print('init_server')
    host = '10.130.64.132'
    port = 4012
    dc = await AioDispatchClient.create(redis_host=host, redis_port=port)



from enum import Enum


class DispatchRequestType(str, Enum):
    JSON = 'JSON'


class DispatchResponseStatus(str, Enum):
    OK = 'OK'
    TIMEOUT = 'TIMEOUT'
    BACKEND_ERROR = 'BACKEND_ERROR'
    NO_RESPONSE_ERROR = 'NO_RESPONSE_ERROR'



import json

from rosetta_dispatcher.model.dispatch_request_model import DispatchRequestModel
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel

reply_to = "a946e098-7674-11eb-8ee6-00163e140fb1"
correlation_id = "a946e099-7674-11eb-8ee6-00163e140fb1"

data = []
data = {'source_language': 'zh',
        'target_language': 'en',
        'chapter_content': [],
        'model_version': '1.0.0'}

response = DispatchResponseModel(correlation_id=correlation_id, data=json.dumps(data))
strresponse = json.dumps(response.dict(), ensure_ascii=True)

print(f'rpush {reply_to} \'{strresponse}\'')



class Singleton(type):
    """
    An metaclass for singleton purpose. Every singleton class should inherit from this class by 'metaclass=Singleton'.
    """
    _instances = {}

    def __call__(cls, *args, **kwargs):
        key = (args, tuple(sorted(kwargs.items())))
        if cls not in cls._instances:
            cls._instances[cls] = {}
        if key not in cls._instances[cls]:
            cls._instances[cls][key] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls][key]




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rosetta-dispatcher",# Replace with your own username
    version="0.0.24",
    author="Walker Wang",
    author_email="walkerwk@foxmail.com",
    description="batch worker client/server via redis queue.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)